@@ -23,9 +23,11 @@
 #include "common.h"
 
 #include <assert.h>
+#include <stddef.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <errno.h>
 #include <string.h>
 #include <ctype.h>
 #include <pwd.h>
@@ -42,7 +44,7 @@ flushn( void )
 	}
 }
 
-static void
+static void ATTR_PRINTFLIKE(1, 0)
 printn( const char *msg, va_list va )
 {
 	if (*msg == '\v')
@@ -149,25 +151,34 @@ error( const char *msg, ... )
 }
 
 void
-sys_error( const char *msg, ... )
+vsys_error( const char *msg, va_list va )
 {
-	va_list va;
 	char buf[1024];
 
+	int errno_bak = errno;
 	flushn();
-	va_start( va, msg );
 	if ((uint)vsnprintf( buf, sizeof(buf), msg, va ) >= sizeof(buf))
 		oob();
-	va_end( va );
+	errno = errno_bak;
 	perror( buf );
 }
 
 void
-add_string_list_n( string_list_t **list, const char *str, int len )
+sys_error( const char *msg, ... )
+{
+	va_list va;
+
+	va_start( va, msg );
+	vsys_error( msg, va );
+	va_end( va );
+}
+
+void
+add_string_list_n( string_list_t **list, const char *str, uint len )
 {
 	string_list_t *elem;
 
-	elem = nfmalloc( sizeof(*elem) + len );
+	elem = nfmalloc( offsetof(string_list_t, string) + len + 1 );
 	elem->next = *list;
 	*list = elem;
 	memcpy( elem->string, str, len );
@@ -203,7 +214,7 @@ vasprintf( char **strp, const char *fmt, va_list ap )
 	if (len >= (int)sizeof(tmp))
 		vsprintf( *strp, fmt, ap );
 	else
-		memcpy( *strp, tmp, len + 1 );
+		memcpy( *strp, tmp, (size_t)len + 1 );
 	return len;
 }
 #endif
@@ -232,34 +243,32 @@ strnlen( const char *str, size_t maxlen )
 #endif
 
 int
-starts_with( const char *str, int strl, const char *cmp, int cmpl )
+starts_with( const char *str, int strl, const char *cmp, uint cmpl )
 {
 	if (strl < 0)
 		strl = strnlen( str, cmpl + 1 );
-	return (strl >= cmpl) && !memcmp( str, cmp, cmpl );
+	return ((uint)strl >= cmpl) && !memcmp( str, cmp, cmpl );
 }
 
 int
-starts_with_upper( const char *str, int strl, const char *cmp, int cmpl )
+starts_with_upper( const char *str, int strl, const char *cmp, uint cmpl )
 {
-	int i;
-
 	if (strl < 0)
 		strl = strnlen( str, cmpl + 1 );
-	if (strl < cmpl)
+	if ((uint)strl < cmpl)
 		return 0;
-	for (i = 0; i < cmpl; i++)
+	for (uint i = 0; i < cmpl; i++)
 		if (str[i] != cmp[i] && toupper( str[i] ) != cmp[i])
 			return 0;
 	return 1;
 }
 
 int
-equals( const char *str, int strl, const char *cmp, int cmpl )
+equals( const char *str, int strl, const char *cmp, uint cmpl )
 {
 	if (strl < 0)
 		strl = strnlen( str, cmpl + 1 );
-	return (strl == cmpl) && !memcmp( str, cmp, cmpl );
+	return ((uint)strl == cmpl) && !memcmp( str, cmp, cmpl );
 }
 
 #ifndef HAVE_TIMEGM
@@ -341,13 +350,13 @@ nfsnprintf( char *buf, int blen, const char *fmt, ... )
 	va_list va;
 
 	va_start( va, fmt );
-	if (blen <= 0 || (uint)(ret = vsnprintf( buf, blen, fmt, va )) >= (uint)blen)
+	if (blen <= 0 || (uint)(ret = vsnprintf( buf, (size_t)blen, fmt, va )) >= (uint)blen)
 		oob();
 	va_end( va );
 	return ret;
 }
 
-static void ATTR_NORETURN
+void
 oom( void )
 {
 	fputs( "Fatal: Out of memory\n", stderr );
@@ -450,20 +459,20 @@ expand_strdup( const char *s )
 	if (*s == '~') {
 		s++;
 		if (!*s) {
-			p = 0;
+			p = NULL;
 			q = Home;
 		} else if (*s == '/') {
 			p = s;
 			q = Home;
 		} else {
 			if ((p = strchr( s, '/' ))) {
-				r = nfstrndup( s, (int)(p - s) );
+				r = nfstrndup( s, (size_t)(p - s) );
 				pw = getpwnam( r );
 				free( r );
 			} else
 				pw = getpwnam( s );
 			if (!pw)
-				return 0;
+				return NULL;
 			q = pw->pw_dir;
 		}
 		nfasprintf( &r, "%s%s", q, p ? p : "" );
@@ -474,10 +483,10 @@ expand_strdup( const char *s )
 
 /* Return value: 0 = ok, -1 = out found in arg, -2 = in found in arg but no out specified */
 int
-map_name( const char *arg, char **result, int reserve, const char *in, const char *out )
+map_name( const char *arg, char **result, uint reserve, const char *in, const char *out )
 {
 	char *p;
-	int i, l, ll, num, inl, outl;
+	uint i, l, ll, num, inl, outl;
 
 	assert( arg );
 	l = strlen( arg );
@@ -491,7 +500,7 @@ map_name( const char *arg, char **result, int reserve, const char *in, const cha
 	}
 	assert( out );
 	outl = strlen( out );
-	if (equals( in, inl, out, outl ))
+	if (equals( in, (int)inl, out, outl ))
 		goto copy;
 	for (num = 0, i = 0; i < l; ) {
 		for (ll = 0; ll < inl; ll++)
@@ -534,7 +543,7 @@ map_name( const char *arg, char **result, int reserve, const char *in, const cha
 static int
 compare_uints( const void *l, const void *r )
 {
-	uint li = *(uint *)l, ri = *(uint *)r;
+	uint li = *(const uint *)l, ri = *(const uint *)r;
 	if (li != ri)  // Can't subtract, the result might not fit into signed int.
 		return li > ri ? 1 : -1;
 	return 0;
@@ -549,16 +558,16 @@ sort_uint_array( uint_array_t array )
 int
 find_uint_array( uint_array_t array, uint value )
 {
-	int bot = 0, top = array.size - 1;
-	while (bot <= top) {
-		int i = (bot + top) / 2;
+	uint bot = 0, top = array.size;
+	while (bot < top) {
+		uint i = (bot + top) / 2;
 		uint elt = array.data[i];
 		if (elt == value)
 			return 1;
 		if (elt < value)
 			bot = i + 1;
 		else
-			top = i - 1;
+			top = i;
 	}
 	return 0;
 }
@@ -585,7 +594,7 @@ arc4_init( void )
 	close( fd );
 
 	for (i = 0; i < 256; i++)
-		rs.s[i] = i;
+		rs.s[i] = (uchar)i;
 	for (i = j = 0; i < 256; i++) {
 		si = rs.s[i];
 		j += si + dat[i & 127];
@@ -617,13 +626,13 @@ static const uchar prime_deltas[] = {
     1, 29,  3, 21,  7, 17, 15,  9, 43, 35, 15,  0,  0,  0,  0,  0
 };
 
-int
-bucketsForSize( int size )
+uint
+bucketsForSize( uint size )
 {
-	int base = 4, bits = 2;
+	uint base = 4, bits = 2;
 
 	for (;;) {
-		int prime = base + prime_deltas[bits];
+		uint prime = base + prime_deltas[bits];
 		if (prime >= size)
 			return prime;
 		base <<= 1;
@@ -651,14 +660,14 @@ list_unlink( list_head_t *head )
 	assert( head->prev->next == head);
 	head->next->prev = head->prev;
 	head->prev->next = head->next;
-	head->next = head->prev = 0;
+	head->next = head->prev = NULL;
 }
 
 static notifier_t *notifiers;
 static int changed;  /* Iterator may be invalid now. */
 #ifdef HAVE_SYS_POLL_H
 static struct pollfd *pollfds;
-static int npolls, rpolls;
+static uint npolls, rpolls;
 #else
 # ifdef HAVE_SYS_SELECT_H
 #  include <sys/select.h>
@@ -669,7 +678,7 @@ void
 init_notifier( notifier_t *sn, int fd, void (*cb)( int, void * ), void *aux )
 {
 #ifdef HAVE_SYS_POLL_H
-	int idx = npolls++;
+	uint idx = npolls++;
 	if (rpolls < npolls) {
 		rpolls = npolls;
 		pollfds = nfrealloc( pollfds, npolls * sizeof(*pollfds) );
@@ -688,28 +697,38 @@ init_notifier( notifier_t *sn, int fd, void (*cb)( int, void * ), void *aux )
 }
 
 void
-conf_notifier( notifier_t *sn, int and_events, int or_events )
+conf_notifier( notifier_t *sn, short and_events, short or_events )
 {
 #ifdef HAVE_SYS_POLL_H
-	int idx = sn->index;
+	uint idx = sn->index;
 	pollfds[idx].events = (pollfds[idx].events & and_events) | or_events;
 #else
 	sn->events = (sn->events & and_events) | or_events;
 #endif
 }
 
+short
+notifier_config( notifier_t *sn )
+{
+#ifdef HAVE_SYS_POLL_H
+	return pollfds[sn->index].events;
+#else
+	return sn->events;
+#endif
+}
+
 void
 wipe_notifier( notifier_t *sn )
 {
 	notifier_t **snp;
 #ifdef HAVE_SYS_POLL_H
-	int idx;
+	uint idx;
 #endif
 
 	for (snp = &notifiers; *snp != sn; snp = &(*snp)->next)
 		assert( *snp );
 	*snp = sn->next;
-	sn->next = 0;
+	sn->next = NULL;
 	changed = 1;
 
 #ifdef HAVE_SYS_POLL_H
@@ -725,7 +744,7 @@ wipe_notifier( notifier_t *sn )
 static time_t
 get_now( void )
 {
-	return time( 0 );
+	return time( NULL );
 }
 
 static list_head_t timers = { &timers, &timers };
@@ -735,7 +754,7 @@ init_wakeup( wakeup_t *tmr, void (*cb)( void * ), void *aux )
 {
 	tmr->cb = cb;
 	tmr->aux = aux;
-	tmr->links.next = tmr->links.prev = 0;
+	tmr->links.next = tmr->links.prev = NULL;
 }
 
 void
@@ -806,7 +825,7 @@ event_wait( void )
 		break;
 	}
 	for (sn = notifiers; sn; sn = sn->next) {
-		int n = sn->index;
+		uint n = sn->index;
 		if ((m = pollfds[n].revents)) {
 			assert( !(m & POLLNVAL) );
 			sn->cb( m | shifted_bit( m, POLLHUP, POLLIN ), sn->aux );
